<?php

require_once('TinkoffMerchantAPI.php');

class Payment_tinkoff_model extends Diafan
{
    /**
     * Формирует данные для платежной формы ТКС
     *
     * @param array $params настройки платежной системы
     * @param array $pay данные о платеже
     * @return array
     */
    public function get($params, $pay)
    {
        if (!isset($params['taxation'])) {
            $params['taxation'] = 'error';
        }

        if ($params['taxation'] == 'error' && $params['check_data_tax'] == 1) {
            echo '<div style="padding-top: 50px; text-align: center;"><p class="back">Не удалось получить данные о системе налогообложения. Проверьте настройки.</p><p class="back"><a href="/shop/">Вернуться в магазин</a></p></div>';
            die();
        }

        if ($this->diafan->configmodules('tax_name', 'shop') != 'Без НДС') {
            switch ($this->diafan->configmodules('tax', 'shop')) {
                case 20:
                    $vat = 'vat20';
                    break;
                case 18:
                    $vat = 'vat18';
                    break;
                case 10:
                    $vat = 'vat10';
                    break;
                case 0:
                    $vat = 'vat0';
                    break;
                default:
                    $vat = 'error';
            }
        } else {
            $vat = 'none';
        }

        if ($vat == 'error' && $params['check_data_tax'] == 1) {
            echo '<div style="padding-top: 50px; text-align: center;"><p class="back">Не удалось получить данные о налоге на товар. Проверьте настройки.</p><p class="back"><a href="/shop/">Вернуться в магазин</a></p></div>';
            die();
        }

        $shippingInfo = $pay['details']['delivery'];
        $cartItems = $pay['details']['goods'];
        $items = [];

        foreach ($cartItems as $item) {
            $price = $this->shopPriceFormat($this->diafan->_shop->price_format($item['price']));
            $amount = $this->shopPriceFormat($this->diafan->_shop->price_format($item['summ']));

            $newItem = array(
                'Name' => mb_substr($item['name'], 0, 64),
                'Price' => round($price * 100),
                'Quantity' => round($item['count'], 2),
                'Amount' => round($amount * 100),
                'PaymentMethod' => trim($params['payment_method']),
                'PaymentObject' => trim($params['payment_object']),
                'Tax' => $vat,
            );

            if(isset($params['ffd_12'])) {
                $newItem['MeasurementUnit'] = 'pc';
            }

            array_push($items, $newItem);
        }

        $priceShipping = $this->shopPriceFormat($this->diafan->_shop->price_format($shippingInfo['summ']));

        $shippingItem = array(
            'Name' => mb_substr($shippingInfo['name'], 0, 64),
            'Price' => round($priceShipping * 100),
            'Quantity' => 1,
            'Amount' => round($priceShipping * 100),
            'PaymentMethod' => trim($params['payment_method']),
            'PaymentObject' => 'service',
            'Tax' => $params['shipping_tax'],
        );

        if(isset($params['ffd_12'])) {
            $shippingItem['MeasurementUnit'] = 'pc';
        }

        $isShipping = false;

        if (round($priceShipping * 100) > 0) {
            array_push($items, $shippingItem);
            $isShipping = true;
        }

        $amount = $this->shopPriceFormat($this->diafan->_shop->price_format($pay['summ']));
        $items_balance = $this->balanceAmount($isShipping, $items, $amount);

        $receipt = array(
            'EmailCompany' => mb_substr($params['tinkoff_email_company'],0,64),
            'Email' => $pay['details']['email'],
            'Phone' => $pay['details']['phone'],
            'Taxation' => $params['taxation'],
            'Items' => $items_balance,
        );

        if(isset($params['ffd_12'])) {
            $receipt['FfdVersion'] = '1.2';
        }

        if ($params['check_data_tax'] == 1) {
            $arrFields = array(
                'OrderId' => $pay['element_id'],
                'Amount' => round($amount * 100),
                'DATA' => array('Email' => $pay['details']['email'], 'Connection_type' => 'diafan_2.3',),
                'Receipt' => $receipt,
            );
        } else {
            $arrFields = array(
                'OrderId' => $pay['element_id'],
                'Amount' => round($amount * 100),
                'DATA' => array('Email' => $pay['details']['email'], 'Connection_type' => 'diafan_2.3',),
            );
        }

        if ($params['language'] == 'en') {
            $arrFields['Language'] = 'en';
        }

        $Tinkoff = new TinkoffMerchantAPI($params['tinkoff_terminal'], $params['tinkoff_password']);
        $request = $Tinkoff->buildQuery('Init', $arrFields);
        // Раскомментируй строку ниже, если хочешь записать лог. После просмотра лога, обязательно удали созданный файл (tinkoff.log) и закомментируй строку.
        // $this->logs($arrFields, $request, 'tinkoff.log');
        $request = json_decode($request);
        $result = array();

        if ($request->PaymentURL) {
            $result['url'] = $request->PaymentURL;
            $result['text'] = $pay['text'];
        }

        return $result;
    }

    function shopPriceFormat($price)
    {
        $shopFormat = preg_replace("/[^x\d|*\,]/", "", $price);
        return preg_replace("/[\,]/", ".", $shopFormat);
    }

    function balanceAmount($isShipping, $items, $amount)
    {
        $itemsWithoutShipping = $items;

        if ($isShipping) {
            $shipping = array_pop($itemsWithoutShipping);
        }

        $sum = 0;

        foreach ($itemsWithoutShipping as $item) {
            $sum += $item['Amount'];
        }

        if (isset($shipping)) {
            $sum += $shipping['Amount'];
        }

        $amount = round($amount * 100);

        if ($sum != $amount) {
            $sumAmountNew = 0;
            $difference = $amount - $sum;
            $amountNews = array();

            foreach ($itemsWithoutShipping as $key => $item) {
                $itemsAmountNew = $item['Amount'] + floor($difference * $item['Amount'] / $sum);
                $amountNews[$key] = $itemsAmountNew;
                $sumAmountNew += $itemsAmountNew;
            }

            if (isset($shipping)) {
                $sumAmountNew += $shipping['Amount'];
            }

            if ($sumAmountNew != $amount) {
                $max_key = array_keys($amountNews, max($amountNews))[0];    // ключ макс значения
                $amountNews[$max_key] = max($amountNews) + ($amount - $sumAmountNew);
            }

            foreach ($amountNews as $key => $item) {
                $items[$key]['Amount'] = $amountNews[$key];
            }
        }
        return $items;
    }

    function logs($requestData, $request, $file)
    {
        // log send
        $log = '[' . date('D M d H:i:s Y', time()) . '] ';
        $log .= json_encode($requestData, JSON_UNESCAPED_UNICODE);
        $log .= "\n";
        file_put_contents(dirname(__FILE__) . "\\". $file, $log, FILE_APPEND);

        $log = '[' . date('D M d H:i:s Y', time()) . '] ';
        $log .= $request;
        $log .= "\n";
        file_put_contents(dirname(__FILE__) . "\\". $file, $log, FILE_APPEND);
    }

    /**
     * Генерирует токен
     *
     * @param $password
     * @return string
     */
    public static function getToken($data, $password)
    {
        $data['Password'] = $password;
        ksort($data);

        if (isset($data['Token'])) {
            unset($data['Token']);
        }

        $values = implode('', array_values($data));

        return hash('sha256', $values);
    }

    /**
     * Получает данные о платеже и платежной системе
     *
     * @param $elementId
     * @return mixed
     */

    public function getPaymentByElementId($elementId)
    {
        return DB::query_fetch_array("SELECT ph.*, p.params FROM {payment_history} as ph 
            JOIN {payment} as p on ph.payment_id=p.id
            WHERE ph.element_id = %d and p.payment = 'tinkoff' LIMIT 1", $elementId);
    }

    /**
     * Отправляет письмо админу об изменении статуса заказа
     *
     * @param $order_id
     */
    public function sendEmailToAdmin($order_id)
    {
        Custom::inc('includes/mail.php');

        $order_status = DB::query_fetch_array(
            "SELECT os.* FROM {shop_order_status} AS os WHERE os.status = (SELECT o.status FROM {shop_order} AS o WHERE id=%d) LIMIT 1",
            $order_id
        );

        if ($order_status) {
            $lang_id = $this->diafan->_languages->site;
            $mail = $this->diafan->configmodules("emailconfadmin", 'shop') ? $this->diafan->configmodules("email_admin", 'shop') : EMAIL_CONFIG;
            $subject = str_replace(
                array('%title', '%url'),
                array(TITLE, BASE_URL),
                $this->diafan->configmodules('subject_change_status', 'shop', 0, $lang_id)
            );
            $message = str_replace(
                array('%title', '%url', '%order', '%status'),
                array(TITLE, BASE_URL, $order_id, $order_status["name" . $lang_id]),
                $this->diafan->configmodules('message_change_status', 'shop', 0, $lang_id)
            );

            send_mail($mail, $subject, $message, $mail);
        }
    }
}